==> framework-plugin-low-code-0.7.1.tgz: package/template/src/handlers/render.jsx <==
import * as React from 'react';
import * as _ from 'lodash';
import { ForContext, getComponentRenderList } from './FieldMiddleware/renderer';
import { isScopeSlot } from '../utils/index';
import { observer } from 'mobx-react-lite';

export function getComponentChildren(component, context = {}) {
  const { properties } = component;
  if (!properties) {
    return [];
  }
  const list = Object.values(properties).sort(
    (a, b) => (a['x-index'] || 0) - (b['x-index'] || 0)
  );
  const {
    virtualFields,
    codeContext,
    scopeContext,
    forContext,
    injectContext = {},
    dataContext,
    updateContext,
  } = context;

  return list.map((schema) => {
    return getRenderList({
      key: schema.key,
      componentSchema: schema,
      rootNode: false,
      renderSlot: false,
      virtualFields,
      codeContext,
      scopeContext,
      forContext,
      injectContext,
      context: dataContext,
      updateContext,
    });
  });
}

function getRenderList(props) {
  const {
    key = '',
    className,
    virtualFields,
    componentSchema,
    renderSlot,
    rootNode = true,
    codeContext,
    scopeContext = {},
    context = {},
    updateContext,
    forContext = {},
  } = props;

  const { 'x-props': xProps, properties = {} } = componentSchema;

  // 判断是否为 slot
  const isSlot = !xProps;
  if (isSlot && !(renderSlot || rootNode)) {
    return null;
  }

  // const preClassName = useRef();

  // wrapperClass
  const containerEl = Object.values(properties)[0];
  if (containerEl && containerEl['x-props'] && className) {
    let { classNameList = [] } = containerEl['x-props'];

    // 先替换掉先前计算出来的className部分
    // if (preClassName.current) {
    //   if (preClassName.current !== className) {
    //     classNameList = classNameList.filter(
    //       (clsName) => clsName !== preClassName.current
    //     );
    //     preClassName.current = className;
    //   }
    // } else {
    //   preClassName.current = className;
    // }

    containerEl['x-props'].classNameList = Array.from(
      new Set([className, ...classNameList])
    );
  }

  if (xProps && xProps.sourceKey) {
    const { sourceKey } = xProps;
    const Field = virtualFields[sourceKey];
    if (!Field) {
      return (
        <div style={{ color: 'red' }}>
          组件<em>{sourceKey}</em>未找到
        </div>
      );
    }
  }

  return getComponentRenderList({
    key,
    componentSchema,
    id: componentSchema.key,
    xProps,
    emitEvents: componentSchema.emitEvents || [],
    virtualFields,
    renderSlot,
    codeContext,
    scopeContext,
    forContext,
    context,
    updateContext,
  });
}

export function generateSlotMetaMap(componentSchema, context, options = {}) {
  const slots = {};
  const { properties = {} } = componentSchema;
  const {
    scopeContext,
    codeContext,
    virtualFields,
    dataContext,
    updateContext,
    forContext,
  } = context;
  // eslint-disable-next-line guard-for-in
  for (const key in properties) {
    const child = properties[key];
    if (!child['x-props'] && child.properties) {
      const isHOC = isScopeSlot(componentSchema, key);

      slots[key] = {
        type: isHOC ? 'HOC' : 'ELEMENT',
        node: isHOC
          ? (props) => {
              let clonedScopeContext = _.cloneDeep(scopeContext);
              _.set(clonedScopeContext, `${componentSchema.key}.${key}`, props);
              return (
                <AppRender
                  key={key}
                  componentSchema={child}
                  renderSlot={options?.renderSlot}
                  virtualFields={virtualFields}
                  codeContext={codeContext}
                  scopeContext={clonedScopeContext}
                  context={dataContext}
                  updateContext={updateContext}
                />
              );
            }
          : () => {
              return (
                <ForContext.Provider value={forContext}>
                  <AppRender
                    key={key}
                    componentSchema={child}
                    renderSlot={options?.renderSlot}
                    virtualFields={virtualFields}
                    codeContext={codeContext}
                    scopeContext={scopeContext}
                    context={dataContext}
                    updateContext={updateContext}
                  />
                </ForContext.Provider>
              );
            },
      };
    }
  }
  return slots;
}

export const AppRender = observer(function (props) {
  return getRenderList({
    ...props,
    forContext: React.useContext(ForContext),
    injectContext: {},
  });
});

==> framework-plugin-low-code-0.7.10.tgz: package/template/src/handlers/render.jsx <==
import * as React from 'react';
import { set, cloneDeep } from 'lodash-es';
import { ForContext, getComponentRenderList } from './FieldMiddleware/renderer';
import { isScopeSlot } from '../utils/index';
import { observer } from 'mobx-react-lite';

export function getComponentChildren(component, context = {}) {
  const { properties } = component;
  if (!properties) {
    return [];
  }
  const list = Object.values(properties).sort((a, b) => (a['x-index'] || 0) - (b['x-index'] || 0));
  const {
    virtualFields,
    codeContext,
    scopeContext,
    forContext,
    injectContext = {},
    dataContext,
    updateContext,
  } = context;

  return list.map((schema) => {
    return getRenderList({
      key: schema.key,
      componentSchema: schema,
      rootNode: false,
      renderSlot: false,
      virtualFields,
      codeContext,
      scopeContext,
      forContext,
      injectContext,
      context: dataContext,
      updateContext,
    });
  });
}

function getRenderList(props) {
  const {
    key = '',
    className,
    virtualFields,
    componentSchema,
    renderSlot,
    rootNode = true,
    codeContext,
    scopeContext = {},
    context = {},
    updateContext,
    forContext = {},
  } = props;

  const { 'x-props': xProps, properties = {} } = componentSchema;

  // 判断是否为 slot
  const isSlot = !xProps;
  if (isSlot && !(renderSlot || rootNode)) {
    return null;
  }

  // const preClassName = useRef();

  // wrapperClass
  const containerEl = Object.values(properties)[0];
  if (containerEl && containerEl['x-props'] && className) {
    let { classNameList = [] } = containerEl['x-props'];

    // 先替换掉先前计算出来的className部分
    // if (preClassName.current) {
    //   if (preClassName.current !== className) {
    //     classNameList = classNameList.filter(
    //       (clsName) => clsName !== preClassName.current
    //     );
    //     preClassName.current = className;
    //   }
    // } else {
    //   preClassName.current = className;
    // }

    containerEl['x-props'].classNameList = Array.from(new Set([className, ...classNameList]));
  }

  if (xProps && xProps.sourceKey) {
    const { sourceKey } = xProps;
    const Field = virtualFields[sourceKey];
    if (!Field) {
      return (
        <div style={{ color: 'red' }}>
          组件<em>{sourceKey}</em>未找到
        </div>
      );
    }
  }

  return getComponentRenderList({
    key,
    componentSchema,
    id: componentSchema.key,
    xProps,
    emitEvents: componentSchema.emitEvents || [],
    virtualFields,
    renderSlot,
    codeContext,
    scopeContext,
    forContext,
    context,
    updateContext,
  });
}

export function generateSlotMetaMap(componentSchema, context, options = {}) {
  const slots = {};
  const { properties = {} } = componentSchema;
  const { scopeContext, codeContext, virtualFields, dataContext, updateContext, forContext } = context;
  // eslint-disable-next-line guard-for-in
  for (const key in properties) {
    const child = properties[key];
    if (!child['x-props'] && child.properties) {
      const isHOC = isScopeSlot(componentSchema, key);

      slots[key] = {
        type: isHOC ? 'HOC' : 'ELEMENT',
        node: isHOC
          ? (props) => {
              let clonedScopeContext = cloneDeep(scopeContext);
              set(clonedScopeContext, `${componentSchema.key}.${key}`, props);
              return (
                <AppRender
                  key={key}
                  componentSchema={child}
                  renderSlot={options?.renderSlot}
                  virtualFields={virtualFields}
                  codeContext={codeContext}
                  scopeContext={clonedScopeContext}
                  context={dataContext}
                  updateContext={updateContext}
                />
              );
            }
          : () => {
              return (
                <ForContext.Provider value={forContext}>
                  <AppRender
                    key={key}
                    componentSchema={child}
                    renderSlot={options?.renderSlot}
                    virtualFields={virtualFields}
                    codeContext={codeContext}
                    scopeContext={scopeContext}
                    context={dataContext}
                    updateContext={updateContext}
                  />
                </ForContext.Provider>
              );
            },
      };
    }
  }
  return slots;
}

export const AppRender = observer(function (props) {
  return getRenderList({
    ...props,
    forContext: React.useContext(ForContext),
    injectContext: {},
  });
});
